@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21058 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-09-18 07:21:37 +0000 (Thu, 18 Sep 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _PSOCKBUN_H
-#define _PSOCKBUN_H
+#ifndef PTLIB_PSOCKBUN_H
+#define PTLIB_PSOCKBUN_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -84,8 +84,12 @@ class PInterfaceMonitor : public PProcessStartup
     /// Change whether the monitor thread should run
     void SetRunMonitorThread (bool runMonitorThread);
 
-    /// Start monitoring network interfaces
-    bool Start();
+    /** Start monitoring network interfaces.
+        If the monitoring thread is already running, then this will cause an
+        refresh of the interface list as soon as possible. Note that this will
+        happen asynchronously.
+      */
+    void Start();
 
     /// Stop monitoring network interfaces.
     void Stop();
@@ -133,7 +137,7 @@ class PInterfaceMonitor : public PProcessStartup
         handles deletion of the filter.
       */
     void SetInterfaceFilter(PInterfaceFilter * filter);
-    bool HasInterfaceFilter() const { return interfaceFilter != NULL; }
+    bool HasInterfaceFilter() const { return m_interfaceFilter != NULL; }
     
     virtual void RefreshInterfaceList();
     
@@ -152,16 +156,20 @@ class PInterfaceMonitor : public PProcessStartup
     typedef PSmartPtr<PInterfaceMonitorClient> ClientPtr;
 
     typedef std::list<PInterfaceMonitorClient *> ClientList_T;
-    ClientList_T              currentClients;
-    PIPSocket::InterfaceTable currentInterfaces;
+    ClientList_T m_clients;
+    PMutex       m_clientsMutex;
 
-    bool           runMonitorThread;
-    PTimeInterval  refreshInterval;
-    PMutex         mutex;
-    PThread      * updateThread;
-    PSyncPoint     threadRunning;
+    PIPSocket::InterfaceTable m_interfaces;
+    PMutex                    m_interfacesMutex;
+
+    bool           m_runMonitorThread;
+    PTimeInterval  m_refreshInterval;
+    PMutex         m_threadMutex;
+    PThread      * m_updateThread;
+    PSyncPoint     m_signalUpdate;
+    bool           m_threadRunning;
     
-    PInterfaceFilter * interfaceFilter;
+    PInterfaceFilter * m_interfaceFilter;
 
   friend class PInterfaceMonitorClient;
 };
@@ -685,4 +693,8 @@ class PSingleMonitoredSocket : public PMonitoredSockets
     SocketInfo     theInfo;
 };
 
-#endif
+
+#endif // PTLIB_PSOCKBUN_H
+
+
+// End Of File ///////////////////////////////////////////////////////////////
