@@ -26,20 +26,20 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21707 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-12-05 07:05:10 +0000 (Fri, 05 Dec 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _CONTAIN_H
+#ifndef PTLIB_CONTAIN_H
 #ifndef _WIN32_WCE
 #error "Please remove pwlib\include\ptlib\msos from the tool include path" \
 "and from the pre-processor options for this project"
 #endif // !_WIN32_WCE
 #endif
 
-#ifndef _OBJECT_H
-#define _OBJECT_H
+#ifndef PTLIB_MSOS_CONTAIN_H
+#define PTLIB_MSOS_CONTAIN_H
 
 
 #ifdef _MSC_VER
@@ -76,7 +76,11 @@
 ///////////////////////////////////////////////////////////////////////////////
 // Machine & Compiler dependent declarations
 
-#if (defined(WIN32) || defined(_WIN32_WCE)) && !defined(_WIN32)
+#ifndef WIN32
+  #define WIN32  1
+#endif
+
+#ifndef _WIN32
   #define _WIN32  1
 #endif
 
@@ -453,7 +457,7 @@ class RegistryKey
 #endif
 
 
-#endif // _OBJECT_H
+#endif // PTLIB_MSOS_CONTAIN_H
 
 
 // End Of File ///////////////////////////////////////////////////////////////
