@@ -26,13 +26,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21161 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-09-25 03:25:07 +0000 (Thu, 25 Sep 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
-#ifndef _POBJECT_H
-#define _POBJECT_H
+#ifndef PTLIB_OBJECT_H
+#define PTLIB_OBJECT_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -107,8 +107,8 @@
 #define PAssert2(b, c, m) (b)
 #define PAssertOS(b) (b)
 #define PAssertNULL(p) (p)
-#define PAssertAlways(m)
-#define PAssertAlways2(c, m)
+#define PAssertAlways(m) {}
+#define PAssertAlways2(c, m) {}
 
 #else // P_USE_ASSERTS
 
@@ -1473,6 +1473,8 @@ PI_SAME(PFloat80b, long double);
  */
 #define PABS(v) ((v) < 0 ? -(v) : (v))
 
-#endif // _POBJECT_H
+
+#endif // PTLIB_OBJECT_H
+
 
 // End Of File ///////////////////////////////////////////////////////////////
