@@ -23,13 +23,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21788 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
  
-#ifndef _SAFE_COLLECTION_H
-#define _SAFE_COLLECTION_H
+#ifndef PTLIB_SAFE_COLLECTION_H
+#define PTLIB_SAFE_COLLECTION_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -240,9 +240,11 @@ class PSafeObject : public PObject
   private:
     mutable PMutex    safetyMutex;
     unsigned          safeReferenceCount;
-    PBoolean              safelyBeingRemoved;
+    bool              safelyBeingRemoved;
     PReadWriteMutex   safeInUseMutex;
     PReadWriteMutex * safeInUse;
+
+  friend class PSafeCollection;
 };
 
 
@@ -1009,7 +1011,7 @@ template <class Key, class Base> class PSafeDictionary : public PSafeDictionaryB
 };
 
 
-#endif // _SAFE_COLLECTION_H
+#endif // PTLIB_SAFE_COLLECTION_H
 
 
-/////////////////////////////////////////////////////////////////////////////
+// End Of File ///////////////////////////////////////////////////////////////
