@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21400 $
+ * $Revision: 21390 $
  * $Author: csoutheren $
- * $Date: 2008-10-21 08:57:27 +0000 (Tue, 21 Oct 2008) $
+ * $Date: 2008-10-21 04:42:44 +0000 (Tue, 21 Oct 2008) $
  */
 
 #ifdef __GNUC__
@@ -230,10 +230,10 @@ PBoolean PVideoInputDevice_YUVFile::SetColourFormat(const PString & newFormat)
 PBoolean PVideoInputDevice_YUVFile::SetFrameRate(unsigned rate)
 {
   // if the file does not know what frame rate it is, then set it
-  if ((file != NULL) && (file->IsUnknownFrameSize() || !file->SetFrameRate(rate)))
+  if (file == NULL || file->IsUnknownFrameSize() || !file->SetFrameRate(rate))
     return PFalse;
 
-  return PVideoDevice::SetFrameRate(rate);
+  return PVideoDevice::SetFrameRate(file->GetFrameRate());
 }
 
 
