@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20892 $
- * $Author: rjongbloed $
- * $Date: 2008-09-08 04:40:50 +0000 (Mon, 08 Sep 2008) $
+ * $Revision: 21708 $
+ * $Author: csoutheren $
+ * $Date: 2008-12-07 23:47:50 +0000 (Sun, 07 Dec 2008) $
  */
 
 #ifdef __GNUC__
@@ -37,6 +37,7 @@
 
 #include <ptlib.h>
 #include <ptlib/pfactory.h>
+#include <ptlib/pprocess.h>
 #include <sstream>
 #include <ctype.h>
 #ifdef _WIN32
@@ -75,6 +76,12 @@ PFactoryBase::FactoryMap::~FactoryMap()
   }  
 }
 
+#if !P_USE_ASSERTS
+
+#define PAssertFunc(a, b, c, d) { }
+
+#else
+
 void PAssertFunc(const char * file,
                  int line,
                  const char * className,
@@ -140,6 +147,8 @@ void PAssertFunc(const char * file, int line, const char * className, const char
   PAssertFunc(str.str().c_str());
 }
 
+#endif // !P_USE_ASSERTS
+
 PObject::Comparison PObject::CompareObjectMemoryDirect(const PObject&obj) const
 {
   int retval = memcmp(this, &obj, sizeof(PObject));
@@ -372,8 +381,7 @@ PMemoryHeap::~PMemoryHeap()
 
 #if defined(_WIN32)
   DeleteCriticalSection(&mutex);
-  extern void PWaitOnExitConsoleWindow();
-  PWaitOnExitConsoleWindow();
+  PProcess::Current().WaitOnExitConsoleWindow();
 #elif defined(P_PTHREADS)
   pthread_mutex_destroy(&mutex);
 #elif defined(P_VXWORKS)
@@ -811,12 +819,6 @@ PMemoryHeap::PMemoryHeap()
 PMemoryHeap::~PMemoryHeap()
 {
   _CrtMemDumpAllObjectsSince(&initialState);
-
-  if (hadCrtDumpLeak) {
-    extern void PWaitOnExitConsoleWindow();
-    PWaitOnExitConsoleWindow();
-  }
-
   _CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) & ~_CRTDBG_LEAK_CHECK_DF);
 }
 
