@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20743 $
- * $Author: hfriederich $
- * $Date: 2008-08-27 17:18:21 +0200 (mer 27 aoû 2008) $
+ * $Revision: 21788 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-12 05:42:13 +0000 (Fri, 12 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -86,7 +86,10 @@ static PIPSocket::Address broadcast4(INADDR_BROADCAST);
 static PIPSocket::Address any4(INADDR_ANY);
 static in_addr inaddr_empty;
 #if P_HAS_IPV6
+static int defaultIPv6ScopeId = 0; 
+
 static PIPSocket::Address loopback6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\001");
+static PIPSocket::Address broadcast6(16,(const BYTE *)"\377\002\0\0\0\0\0\0\0\0\0\0\0\0\0\001"); // IPV6 multicast address
 static PIPSocket::Address any6(16,(const BYTE *)"\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0"); 
 #endif
 
@@ -116,6 +119,15 @@ void PIPSocket::SetDefaultIpAddressFamilyV6()
   SetDefaultIpAddressFamily(PF_INET6);
 }
 
+void PIPSocket::SetDefaultV6ScopeId(int scopeId) 
+{
+  defaultIPv6ScopeId = scopeId;
+}; 
+
+int PIPSocket::GetDefaultV6ScopeId()
+{
+  return defaultIPv6ScopeId;
+}
 
 PBoolean PIPSocket::IsIpAddressFamilyV6Supported()
 {
@@ -172,7 +184,7 @@ Psockaddr::Psockaddr(const PIPSocket::Address & ip, WORD port)
     addr6->sin6_addr = ip;
     addr6->sin6_port = htons(port);
     addr6->sin6_flowinfo = 0;
-    addr6->sin6_scope_id = 0; // Should be set to the right interface....
+	addr6->sin6_scope_id = PIPSocket::GetDefaultV6ScopeId(); // Should be set to the right interface....
   }
   else {
     sockaddr_in * addr4 = (sockaddr_in *)&storage;
@@ -488,14 +500,23 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
 {
   mutex.Wait();
 
-  PCaselessString key = name;
+  PString key = name;
+  PINDEX len = key.GetLength();
 
   // Check for a legal hostname as per RFC952
+  // but drop the requirement for leading alpha as per RFC 1123
   if (key.IsEmpty() ||
-      key.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-.") != P_MAX_INDEX ||
-      key[key.GetLength()-1] == '-' ||
-      !isalpha(key[0]))
+      key.FindSpan("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.") != P_MAX_INDEX ||
+      key[len-1] == '-') {
+    PTRACE(3, "Socket\tIllegal RFC952 characters in DNS name \"" << key << '"');
     return NULL;
+  }
+
+  // We lowercase this way rather than toupper() as that is locale dependent, and DNS names aren't.
+  for (PINDEX i = 0; i < len; i++) {
+    if (key[i] >= 'a')
+      key[i] &= 0x5f;
+  }
 
   PIPCacheData * host = GetAt(key);
   int localErrNo = NO_DATA;
@@ -587,12 +608,11 @@ PIPCacheData * PHostByName::GetHost(const PString & name)
     SetAt(key, host);
   }
 
-  if (host->GetHostAddress() == 0) {
-    PTRACE(5, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
-    return NULL;
-  }
+  if (host->GetHostAddress().IsValid())
+    return host;
 
-  return host;
+  PTRACE(4, "Socket\tName lookup of \"" << name << "\" failed: errno=" << localErrNo);
+  return NULL;
 }
 
 
@@ -723,10 +743,7 @@ PIPCacheData * PHostByAddr::GetHost(const PIPSocket::Address & addr)
     SetAt(key, host);
   }
 
-  if (host->GetHostAddress() == 0)
-    return NULL;
-
-  return host;
+  return host->GetHostAddress().IsValid() ? host : NULL;
 }
 
 
@@ -1118,7 +1135,7 @@ PString PIPSocket::GetHostName(const PString & hostname)
 {
   // lookup the host address using inet_addr, assuming it is a "." address
   Address temp = hostname;
-  if (temp != 0)
+  if (temp.IsValid())
     return GetHostName(temp);
 
   PString canonicalname;
@@ -1131,13 +1148,18 @@ PString PIPSocket::GetHostName(const PString & hostname)
 
 PString PIPSocket::GetHostName(const Address & addr)
 {
-  if (addr == 0)
+  if (!addr.IsValid())
     return addr.AsString();
 
   PString hostname;
   if (pHostByAddr().GetHostName(addr, hostname))
     return hostname;
 
+#if P_HAS_IPV6
+  if (addr.GetVersion() == 6)
+    return '[' + addr.AsString() + ']';
+#endif
+
   return addr.AsString();
 }
 
@@ -1177,7 +1199,7 @@ PStringArray PIPSocket::GetHostAliases(const PString & hostname)
 
   // lookup the host address using inet_addr, assuming it is a "." address
   Address addr = hostname;
-  if (addr != 0)
+  if (addr.IsValid())
     pHostByAddr().GetHostAliases(addr, aliases);
   else
     pHostByName().GetHostAliases(hostname, aliases);
@@ -1453,12 +1475,6 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 
 #if P_HAS_IPV6
   Psockaddr bind_sa(bindAddr, port); 
-
-  if (IsOpen()) {
-    int socketType;
-    if (!GetOption(SO_TYPE, socketType, SOL_SOCKET) || bind_sa->sa_family != socketType)
-      Close();
-  }
 #endif
 
   if (!IsOpen()) {
@@ -1472,12 +1488,17 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 #endif
   }
   
-#ifndef __BEOS__
-  // attempt to listen
-  if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
-    os_close();
-    return PFalse;
+#ifndef P_BEOS
+#if P_HAS_IPV6
+  if(bind_sa->sa_family != AF_INET6)
+  {
+	 // attempt to listen
+	 if (!SetOption(SO_REUSEADDR, reuse == CanReuseAddress ? 1 : 0)) {
+	   os_close();
+	   return PFalse;
+	 }
   }
+#endif
 #else
   // attempt to listen
   int value = reuse == CanReuseAddress ? 1 : 0;
@@ -1533,12 +1554,6 @@ PBoolean PIPSocket::Listen(const Address & bindAddr,
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetLoopback()
-{
-  return loopback4;
-}
-
-
 #if P_HAS_IPV6
 
 /// Check for v4 mapped i nv6 address ::ffff:a.b.c.d
@@ -1550,15 +1565,36 @@ PBoolean PIPSocket::Address::IsV4Mapped() const
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetLoopback6()
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int version)
 {
-  return loopback6;
+  return version == 6 ? loopback6 : loopback4;
 }
 
-
-const PIPSocket::Address & PIPSocket::Address::GetAny6()
+const PIPSocket::Address & PIPSocket::Address::GetAny(int version)
 {
-  return any6;
+  return version == 6 ? any6 : any4;
+}
+
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int version)
+{
+  return version == 6 ? broadcast6 : broadcast4;
+}
+
+#else
+
+const PIPSocket::Address & PIPSocket::Address::GetLoopback(int)
+{
+  return loopback4;
+}
+
+const PIPSocket::Address & PIPSocket::Address::GetAny(int)
+{
+  return any4;
+}
+
+const PIPSocket::Address PIPSocket::Address::GetBroadcast(int)
+{
+  return broadcast4;
 }
 
 #endif
@@ -1570,15 +1606,14 @@ PBoolean PIPSocket::Address::IsAny() const
 }
 
 
-const PIPSocket::Address & PIPSocket::Address::GetBroadcast()
-{
-  return broadcast4;
-}
-
-
 PIPSocket::Address::Address()
 {
-  *this = loopback4;
+#if P_HAS_IPV6
+	if(defaultIpAddressFamily == AF_INET6)
+		*this = loopback6;
+	else
+#endif
+		*this = loopback4;
 }
 
 
@@ -1751,72 +1786,24 @@ bool PIPSocket::Address::operator==(in_addr & addr) const
 
 bool PIPSocket::Address::operator==(DWORD dw) const
 {
-  if (dw != 0)
+  if (dw == 0)
+    return !IsValid();
+
+  if (version == 4)
     return (DWORD)*this == dw;
 
-  return !IsValid();
+  return *this == Address(dw);
 }
 
 
 PIPSocket::Address & PIPSocket::Address::operator=(const PString & dotNotation)
 {
-  version = 0;
-  memset(&v, 0, sizeof(v));
-
-#if P_HAS_IPV6
-
-  struct addrinfo *res = NULL;
-  struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
-
-  if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
-    if (res->ai_family == PF_INET6) {
-      // IPv6 addr
-      version = 6;
-      struct sockaddr_in6 * addr_in6 = (struct sockaddr_in6 *)res->ai_addr;
-      v.six = addr_in6->sin6_addr;
-    } else {
-      // IPv4 addr
-      version = 4;
-      struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
-      v.four = addr_in->sin_addr;
-    }
-    freeaddrinfo(res);
-  }
-
-#else //P_HAS_IPV6
-
-  DWORD iaddr;
-  if (dotNotation.FindSpan("0123456789.") == P_MAX_INDEX &&
-                    (iaddr = ::inet_addr((const char *)dotNotation)) != (DWORD)INADDR_NONE) {
-    version = 4;
-    v.four.s_addr = iaddr;
-  }
-
-#endif
-
-  else {
-    PINDEX percent = dotNotation.Find('%');
-    if (percent != P_MAX_INDEX) {
-      PString iface = dotNotation.Mid(percent+1);
-      if (!iface.IsEmpty()) {
-        PIPSocket::InterfaceTable interfaceTable;
-        if (PIPSocket::GetInterfaceTable(interfaceTable)) {
-          for (PINDEX i = 0; i < interfaceTable.GetSize(); i++) {
-            if (interfaceTable[i].GetName().NumCompare(iface) == EqualTo) {
-              *this = interfaceTable[i].GetAddress();
-              break;
-            }
-          }
-        }
-      }
-    }
-  }
-
+  FromString(dotNotation);
   return *this;
 }
 
 
-PString PIPSocket::Address::AsString() const
+PString PIPSocket::Address::AsString(bool bracketIPv6) const
 {
 #if defined(P_VXWORKS)
   char ipStorage[INET_ADDR_LEN];
@@ -1832,7 +1819,9 @@ PString PIPSocket::Address::AsString() const
     if (percent != P_MAX_INDEX)
       str[percent] = '\0';
     str.MakeMinimumSize();
-    return str;
+    if (bracketIPv6)
+      return '[' + str + ']';
+	return str;
   }
 #endif // P_HAS_IPV6
 # if defined(P_HAS_INET_NTOP)
@@ -1850,10 +1839,73 @@ PString PIPSocket::Address::AsString() const
 }
 
 
-PBoolean PIPSocket::Address::FromString(const PString & dotNotation)
+PBoolean PIPSocket::Address::FromString(const PString & ipAndInterface)
 {
-  (*this) = dotNotation;
-  return IsValid();
+  version = 0;
+  memset(&v, 0, sizeof(v));
+
+  PINDEX percent = ipAndInterface.Find('%');
+  PString dotNotation = ipAndInterface.Left(percent);
+  if (!dotNotation.IsEmpty()) {
+#if P_HAS_IPV6
+
+    // Find out if string is in brackets [], as in ipv6 address
+    PINDEX lbracket = dotNotation.Find('[');
+    PINDEX rbracket = dotNotation.Find(']', lbracket);
+    if (lbracket != P_MAX_INDEX && rbracket != P_MAX_INDEX)
+      dotNotation = dotNotation(lbracket+1, rbracket-1);
+
+    struct addrinfo *res = NULL;
+    struct addrinfo hints = { AI_NUMERICHOST, PF_UNSPEC }; // Could be IPv4: x.x.x.x or IPv6: x:x:x:x::x
+
+    if (getaddrinfo((const char *)dotNotation, NULL , &hints, &res) == 0) {
+      if (res->ai_family == PF_INET6) {
+        // IPv6 addr
+        version = 6;
+        struct sockaddr_in6 * addr_in6 = (struct sockaddr_in6 *)res->ai_addr;
+        v.six = addr_in6->sin6_addr;
+      } else {
+        // IPv4 addr
+        version = 4;
+        struct sockaddr_in * addr_in = (struct sockaddr_in *)res->ai_addr;
+        v.four = addr_in->sin_addr;
+      }
+      freeaddrinfo(res);
+      return IsValid();
+    }
+
+#else //P_HAS_IPV6
+
+    DWORD iaddr;
+    if (dotNotation.FindSpan("0123456789.") == P_MAX_INDEX &&
+                      (iaddr = ::inet_addr((const char *)dotNotation)) != (DWORD)INADDR_NONE) {
+      version = 4;
+      v.four.s_addr = iaddr;
+      return true;
+    }
+
+#endif
+  }
+
+  if (percent == P_MAX_INDEX)
+    return false;
+
+  PString iface = ipAndInterface.Mid(percent+1);
+  if (iface.IsEmpty())
+    return false;
+
+  PIPSocket::InterfaceTable interfaceTable;
+  if (!PIPSocket::GetInterfaceTable(interfaceTable))
+    return false;
+
+  for (PINDEX i = 0; i < interfaceTable.GetSize(); i++) {
+    if (interfaceTable[i].GetName().NumCompare(iface) == EqualTo) {
+      *this = interfaceTable[i].GetAddress();
+      return true;
+    }
+  }
+
+  return false;
 
 }
 
@@ -1968,7 +2020,7 @@ PBoolean PIPSocket::Address::IsBroadcast() const
 {
 #if P_HAS_IPV6
   if (version == 6) // In IPv6, no broadcast exist. Only multicast
-    return PFalse;
+    return *this == broadcast6; // multicast address as as substitute
 #endif
 
   return *this == broadcast4;
@@ -2327,7 +2379,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
 
   PBoolean broadcast = addr.IsAny() || addr.IsBroadcast();
   if (broadcast) {
-#ifdef __BEOS__
+#ifdef P_BEOS
     PAssertAlways("Broadcast option under BeOS is not implemented yet");
     return PFalse;
 #else
@@ -2381,7 +2433,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
   
 #if P_HAS_IPV6
   
-  Psockaddr sa(broadcast ? Address::GetBroadcast() : addr, port);
+  Psockaddr sa(broadcast ? Address::GetBroadcast(addr.GetVersion()) : addr, port);
   PBoolean ok = os_sendto(buf, len, 0, sa, sa.GetSize()) != 0;
   
 #else
@@ -2396,7 +2448,7 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
   
 #endif // P_MACOSX
 
-#ifndef __BEOS__
+#ifndef P_BEOS
   if (broadcast)
     SetOption(SO_BROADCAST, 0);
 #endif
@@ -2408,30 +2460,44 @@ PBoolean PIPDatagramSocket::WriteTo(const void * buf, PINDEX len,
 //////////////////////////////////////////////////////////////////////////////
 // PUDPSocket
 
-PUDPSocket::PUDPSocket(WORD newPort)
+PUDPSocket::PUDPSocket(WORD newPort, int iAddressFamily)
 {
   sendPort = 0;
   SetPort(newPort);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
-PUDPSocket::PUDPSocket(PQoS * qos, WORD newPort)
+PUDPSocket::PUDPSocket(PQoS * qos, WORD newPort, int iAddressFamily)
+#if P_HAS_IPV6
+  : sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
+    lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
+#else
+  : sendAddress(loopback4),
+    lastReceiveAddress(loopback4)
+#endif
 {
   if (qos != NULL)
       qosSpec = *qos;
   sendPort = 0;
   SetPort(newPort);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
 
-PUDPSocket::PUDPSocket(const PString & service, PQoS * qos)
+PUDPSocket::PUDPSocket(const PString & service, PQoS * qos, int iAddressFamily)
+#if P_HAS_IPV6
+  : sendAddress(iAddressFamily == AF_INET ? loopback4 : loopback6),
+    lastReceiveAddress(iAddressFamily == AF_INET ? loopback4 : loopback6)
+#else
+  : sendAddress(loopback4),
+    lastReceiveAddress(loopback4)
+#endif
 {
   if (qos != NULL)
       qosSpec = *qos;
   sendPort = 0;
   SetPort(service);
-  OpenSocket();
+  OpenSocket(iAddressFamily);
 }
 
 
@@ -2652,18 +2718,16 @@ PBoolean PUDPSocket::OpenSocketGQOS(int af, int type, int proto)
 
 #define COULD_HAVE_QOS
 
-static PBoolean CheckOSVersion()
+static PBoolean CheckOSVersionFor(DWORD major, DWORD minor)
 {
-    OSVERSIONINFO versInfo;
-    ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
-    versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    if (GetVersionEx(&versInfo))
-    {
-        if (versInfo.dwMajorVersion > 5 ||
-           (versInfo.dwMajorVersion == 5 &&
-            versInfo.dwMinorVersion > 0))
-          return PTrue;
-    }
+  OSVERSIONINFO versInfo;
+  ZeroMemory(&versInfo,sizeof(OSVERSIONINFO));
+  versInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+  if (GetVersionEx(&versInfo)) {
+    if (versInfo.dwMajorVersion > major ||
+       (versInfo.dwMajorVersion == major && versInfo.dwMinorVersion >= minor))
+      return PTrue;
+  }
   return PFalse;
 }
 
@@ -2674,7 +2738,7 @@ static PBoolean CheckOSVersion()
 PBoolean PUDPSocket::OpenSocket()
 {
 #ifdef COULD_HAVE_QOS
-  if (CheckOSVersion()) 
+  if (CheckOSVersionFor(5,1)) 
     return OpenSocketGQOS(AF_INET, SOCK_DGRAM, 0);
 #endif
 
@@ -2684,7 +2748,7 @@ PBoolean PUDPSocket::OpenSocket()
 PBoolean PUDPSocket::OpenSocket(int ipAdressFamily)
 {
 #ifdef COULD_HAVE_QOS
-  if (CheckOSVersion()) 
+  if (CheckOSVersionFor(5,1)) 
     return OpenSocketGQOS(ipAdressFamily, SOCK_DGRAM, 0);
 #endif
 
@@ -2749,23 +2813,23 @@ PBoolean PICMPSocket::OpenSocket(int)
 
 //////////////////////////////////////////////////////////////////////////////
 
-PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD _port, char _sep)
+PBoolean PIPSocketAddressAndPort::Parse(const PString & str, WORD port, char separator)
 {
-  sep  = _sep;
-  port = _port;
+  m_separator= separator;
+  m_port = port;
 
-  PINDEX pos = str.Find(sep);
+  PINDEX pos = str.Find(separator);
   if (pos != P_MAX_INDEX) {
-    port    = (WORD)str.Mid(pos+1).AsInteger();
-    if (!PIPSocket::GetHostAddress(str.Left(pos), address))
-      return PFalse;
+    m_port    = (WORD)str.Mid(pos+1).AsInteger();
+    if (!PIPSocket::GetHostAddress(str.Left(pos), m_address))
+      return false;
   }
-  else if (port == 0) 
-    return PFalse;
-  else if (!PIPSocket::GetHostAddress(str, address))
-    return PFalse;
+  else if (m_port == 0) 
+    return false;
+  else if (!PIPSocket::GetHostAddress(str, m_address))
+    return false;
 
-  return PTrue;
+  return true;
 }
 
 
