@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20385 $
+ * $Revision: 21597 $
  * $Author: rjongbloed $
- * $Date: 2008-06-04 12:40:38 +0200 (mer 04 jun 2008) $
+ * $Date: 2008-11-23 12:25:25 +0000 (Sun, 23 Nov 2008) $
  */
 
 #include <ptlib.h>
@@ -41,6 +41,7 @@
 #endif
 
 #ifndef _WIN32_WCE
+#include <signal.h>
 #include <share.h>
 #endif
 
@@ -980,6 +981,19 @@ PBoolean PConsoleChannel::Close()
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess
 
+static void (__cdecl * PreviousSigIntHandler)(int);
+static void (__cdecl * PreviousSigTermHandler)(int);
+
+#ifndef _WIN32_WCE 
+void SignalHandler(int sig)
+{
+  if (PProcess::Current().OnInterrupt(sig == SIGTERM))
+    return;
+
+  (sig == SIGTERM ? PreviousSigTermHandler : PreviousSigIntHandler)(sig);
+}
+#endif
+
 void PProcess::Construct()
 {
   PSetErrorStream(&cerr);
@@ -993,6 +1007,11 @@ void PProcess::Construct()
 #endif
 
   houseKeeper = NULL;
+
+#ifndef _WIN32_WCE 
+  PreviousSigIntHandler = signal(SIGINT, SignalHandler);
+  PreviousSigTermHandler = signal(SIGTERM, SignalHandler);
+#endif
 }
 
 
