@@ -24,9 +24,9 @@
  * Contributor(s): ______________________________________.
  *                 Walter H Whitlock (twohives@nc.rr.com)
  *
- * $Revision: 21353 $
- * $Author: csoutheren $
- * $Date: 2008-10-16 00:33:55 +0000 (Thu, 16 Oct 2008) $
+ * $Revision: 21764 $
+ * $Author: rjongbloed $
+ * $Date: 2008-12-10 07:44:14 +0000 (Wed, 10 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -38,12 +38,12 @@
 #include <ptlib/pluginmgr.h>
 #include <ptclib/delaychan.h>
 
+#if P_VFW_CAPTURE
+
 #ifdef _MSC_VER
-#ifndef _WIN32_WCE
 #pragma comment(lib, "vfw32.lib")
 #pragma comment(lib, "gdi32.lib")
 #endif
-#endif
 
 namespace PWLibStupidLinkerHacks {
    int loadVideoForWindowsStuff;
@@ -172,6 +172,10 @@ BOOL VFWAPI capGetDriverDescriptionA (WORD wDriverIndex, LPSTR lpszName,
 #define capGetDriverDescription capGetDriverDescriptionA
 #define capCreateCaptureWindow capCreateCaptureWindowA
 
+#else // __MINGW32
+
+  #include <vfw.h>
+
 #endif // __MINGW32
 
 #define STEP_GRAB_CAPTURE 1
@@ -843,8 +847,10 @@ PBoolean PVideoInputDevice_VideoForWindows::GetFrameData(BYTE * buffer, PINDEX *
 
 PBoolean PVideoInputDevice_VideoForWindows::GetFrameDataNoDelay(BYTE * buffer, PINDEX * bytesReturned)
 {
-  if (!frameAvailable.Wait(1000))
+  if (!frameAvailable.Wait(1000)) {
+    PTRACE(1, "PVidInp\tTimeout waiting for frame grab!");
     return PFalse;
+  }
 
   bool retval = false;
 
@@ -1044,6 +1050,8 @@ void PVideoInputDevice_VideoForWindows::HandleCapture(PThread &, INT)
     threadStarted.Signal();
 }
 
+#endif // P_VFW_CAPTURE
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // PVideoOutputDevice_Window
@@ -1160,8 +1168,8 @@ class PVideoOutputDevice_Window : public PVideoOutputDeviceRGB
     PSyncPoint m_started;
     BITMAPINFO m_bitmap;
     bool       m_flipped;
-    int        m_lastX;
-    int        m_lastY;
+    POINT      m_lastPosition;
+    SIZE       m_fixedSize;
 };
 
 
@@ -1225,8 +1233,10 @@ PVideoOutputDevice_Window::PVideoOutputDevice_Window()
   m_hWnd = NULL;
   m_thread = NULL;
   m_flipped = PFalse;
-  m_lastX = 0;
-  m_lastY = 0;
+  m_lastPosition.x = 0;
+  m_lastPosition.y = 0;
+  m_fixedSize.cx = 0;
+  m_fixedSize.cy = 0;
 
   m_bitmap.bmiHeader.biSize = sizeof(m_bitmap.bmiHeader);
   m_bitmap.bmiHeader.biWidth = frameWidth;
@@ -1377,10 +1387,10 @@ void PVideoOutputDevice_Window::SetWindowSize()
   RECT rect;
   rect.top = 0;
   rect.left = 0;
-  rect.bottom = frameHeight;
-  rect.right = frameWidth;
+  rect.bottom = m_fixedSize.cy > 0 ? m_fixedSize.cy : frameHeight;
+  rect.right = m_fixedSize.cx > 0 ? m_fixedSize.cx : frameWidth;
   ::AdjustWindowRectEx(&rect, GetWindowLong(m_hWnd, GWL_STYLE), false, GetWindowLong(m_hWnd, GWL_EXSTYLE));
-  ::SetWindowPos(m_hWnd, NULL, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE|SWP_NOZORDER);
+  ::SetWindowPos(m_hWnd, HWND_TOP, 0, 0, rect.right-rect.left, rect.bottom-rect.top, SWP_NOMOVE);
 }
 
 
@@ -1401,8 +1411,8 @@ PBoolean PVideoOutputDevice_Window::FrameComplete()
 
 PBoolean PVideoOutputDevice_Window::GetPosition(int & x, int & y) const
 {
-  x = m_lastX;
-  y = m_lastY;
+  x = m_lastPosition.x;
+  y = m_lastPosition.y;
   return PTrue;
 }
 
@@ -1444,6 +1454,17 @@ static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lPar
 }
 
 
+static int GetTokenValue(const PString & deviceName, const char * token, int defaultValue)
+{
+  PINDEX pos = deviceName.Find(token);
+  if (pos == P_MAX_INDEX)
+    return defaultValue;
+
+  pos += strlen(token);
+  return deviceName.Mid(pos).AsInteger();
+}
+
+
 void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
 {
 #ifndef _WIN32_WCE
@@ -1479,17 +1500,29 @@ void PVideoOutputDevice_Window::HandleDisplay(PThread &, INT)
       title = PString(PString::Literal, quotedTitle);
     }
 
-    pos = deviceName.Find("X=");
-    m_lastX = pos != P_MAX_INDEX ? atoi(&deviceName[pos+2]) : CW_USEDEFAULT;
+    m_lastPosition.x = GetTokenValue(deviceName, "X=", CW_USEDEFAULT);
+    m_lastPosition.y = GetTokenValue(deviceName, "Y=", CW_USEDEFAULT);
+    m_fixedSize.cx   = GetTokenValue(deviceName, "WIDTH=", 0);
+    m_fixedSize.cy   = GetTokenValue(deviceName, "HEIGHT=", 0);
 
-    pos = deviceName.Find("Y=");
-    m_lastY = pos != P_MAX_INDEX ? atoi(&deviceName[pos+2]) : CW_USEDEFAULT;
+    if (m_lastPosition.x == CW_USEDEFAULT && m_lastPosition.y == CW_USEDEFAULT) {
+      if (hParent != NULL) {
+        RECT rect;
+        GetWindowRect(hParent, &rect);
+        m_lastPosition.x = (rect.right + rect.left - frameWidth)/2;
+        m_lastPosition.y = (rect.bottom + rect.top - frameHeight)/2;
+      }
+      else {
+        m_lastPosition.x = (GetSystemMetrics(SM_CXSCREEN) - frameWidth)/2;
+        m_lastPosition.y = (GetSystemMetrics(SM_CYSCREEN) - frameHeight)/2;
+      }
+    }
 
     PVarString windowTitle = title;
     m_hWnd = CreateWindow(wndClassName,
                           windowTitle, 
                           dwStyle,
-                          m_lastX, m_lastY, frameWidth, frameHeight,
+                          m_lastPosition.x, m_lastPosition.y, frameWidth, frameHeight,
                           hParent, NULL, GetModuleHandle(NULL), this);
     SetWindowSize();
   }
@@ -1522,8 +1555,8 @@ LRESULT PVideoOutputDevice_Window::WndProc(UINT uMsg, WPARAM wParam, LPARAM lPar
       if (m_hWnd != NULL) {
         RECT rect;
         GetWindowRect(m_hWnd, &rect);
-        m_lastX = rect.left;
-        m_lastY = rect.top;
+        m_lastPosition.x = rect.left;
+        m_lastPosition.y = rect.top;
       }
       break;
 
