@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20899 $
+ * $Revision: 21682 $
  * $Author: rjongbloed $
- * $Date: 2008-09-08 07:44:37 +0000 (Mon, 08 Sep 2008) $
+ * $Date: 2008-12-02 23:47:17 +0000 (Tue, 02 Dec 2008) $
  */
 
 #include <ptlib.h>
@@ -729,6 +729,30 @@ void PThread::Win32AttachThreadInput()
 }
 
 
+PThread::PThread()
+  : autoDelete(false)
+  , originalStackSize(0)
+  , threadHandle(GetCurrentThread())
+  , threadId(GetCurrentThreadId())
+{
+  if (!PProcess::IsInitialised())
+    return;
+
+  autoDelete = true;
+  DuplicateHandle(GetCurrentProcess(), threadHandle, GetCurrentProcess(), &threadHandle, 0, 0, DUPLICATE_SAME_ACCESS);
+
+  PProcess & process = PProcess::Current();
+
+  process.activeThreadMutex.Wait();
+  process.activeThreads.SetAt(threadId, this);
+  process.activeThreadMutex.Signal();
+
+  process.deleteThreadMutex.Wait();
+  process.autoDeleteThreads.Append(this);
+  process.deleteThreadMutex.Signal();
+}
+
+
 PThread::PThread(PINDEX stackSize,
                  AutoDeleteFlag deletion,
                  Priority priorityLevel,
@@ -967,27 +991,6 @@ void PThread::Yield()
 }
 
 
-void PThread::InitialiseProcessThread()
-{
-  originalStackSize = 0;
-  autoDelete = PFalse;
-  threadHandle = GetCurrentThread();
-  threadId = GetCurrentThreadId();
-  ((PProcess *)this)->activeThreads.DisallowDeleteObjects();
-  ((PProcess *)this)->activeThreads.SetAt(threadId, this);
-}
-
-
-PThread * PThread::Current()
-{
-  PProcess & process = PProcess::Current();
-  process.activeThreadMutex.Wait();
-  PThread * thread = process.activeThreads.GetAt(GetCurrentThreadId());
-  process.activeThreadMutex.Signal();
-  return thread;
-}
-
-
 ///////////////////////////////////////////////////////////////////////////////
 // PProcess::TimerThread
 
@@ -1113,8 +1116,7 @@ PProcess::~PProcess()
 #endif
 
 #if _DEBUG
-  extern void PWaitOnExitConsoleWindow();
-  PWaitOnExitConsoleWindow();
+  WaitOnExitConsoleWindow();
 #endif
 }
 
