@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20909 $
- * $Author: csoutheren $
- * $Date: 2008-09-09 07:25:48 +0000 (Tue, 09 Sep 2008) $
+ * $Revision: 21624 $
+ * $Author: rjongbloed $
+ * $Date: 2008-11-26 01:24:10 +0000 (Wed, 26 Nov 2008) $
  */
 
 #pragma implementation "sockets.h"
@@ -80,7 +80,7 @@
 #include <bsp.h>
 #endif
 
-#ifdef __BEOS__
+#ifdef P_BEOS
 #include <posix/sys/ioctl.h> // for FIONBIO
 #include <be/bone/net/if.h> // for ifconf
 #include <be/bone/sys/sockio.h> // for SIOCGI*
@@ -223,7 +223,7 @@ PBoolean PSocket::os_accept(PSocket & listener, struct sockaddr * addr, PINDEX *
 }
 
 
-#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS) && !defined(__BEOS__)
+#if !defined(P_PTHREADS) && !defined(P_MAC_MPTHREADS) && !defined(P_BEOS)
 
 PChannel::Errors PSocket::Select(SelectList & read,
                                  SelectList & write,
@@ -983,7 +983,7 @@ PBoolean PEthSocket::Write(const void * buf, PINDEX len)
 
 ///////////////////////////////////////////////////////////////////////////////
 
-PBoolean PIPSocket::GetGatewayAddress(Address & addr)
+PBoolean PIPSocket::GetGatewayAddress(Address & addr, int version)
 {
   RouteTable table;
   if (GetRouteTable(table)) {
@@ -999,7 +999,7 @@ PBoolean PIPSocket::GetGatewayAddress(Address & addr)
 
 
 
-PString PIPSocket::GetGatewayInterface()
+PString PIPSocket::GetGatewayInterface(int version)
 {
   RouteTable table;
   if (GetRouteTable(table)) {
@@ -1029,11 +1029,11 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
     unsigned long net_addr, dest_addr, net_mask;
     int flags, refcnt, use, metric;
     procfile >> iface;
-    procfile.setf(hex, dec);
+    procfile.setf(hex, ios::basefield);
     procfile >> net_addr >> dest_addr >> flags;
-    procfile.setf(dec, hex);
+    procfile.setf(dec, ios::basefield);
     procfile >> refcnt >> use >> metric;
-    procfile.setf(hex, dec);
+    procfile.setf(hex, ios::basefield);
     procfile >> net_mask;
     if (procfile.bad())
       return PFalse;
@@ -1517,6 +1517,17 @@ PBoolean PIPSocket::GetRouteTable(RouteTable & table)
 #endif
 
 
+bool PIPSocket::WaitForRouteTableChange(const PTimeInterval & timeout, PSyncPoint * cancellation)
+{
+  if (cancellation != NULL)
+    return cancellation->Wait(timeout);
+
+  // Need an implementation here!!!!!!!
+  PThread::Sleep(timeout);
+  return false;
+}
+
+
 // fe800000000000000202e3fffe1ee330 02 40 20 80     eth0
 // 00000000000000000000000000000001 01 80 10 80       lo
 
@@ -1602,11 +1613,11 @@ PBoolean PIPSocket::GetInterfaceTable(InterfaceTable & list, PBoolean includeDow
             memcpy(&ifReq, ifName, sizeof(ifreq));
             if (ioctl(sock.GetHandle(), SIOCGIFNETMASK, &ifReq) >= 0) {
               PIPSocket::Address mask = 
-#ifndef __BEOS__
+#ifndef P_BEOS
     ((sockaddr_in *)&ifReq.ifr_netmask)->sin_addr;
 #else
     ((sockaddr_in *)&ifReq.ifr_mask)->sin_addr;
-#endif // !__BEOS__
+#endif // !P_BEOS
               PINDEX i;
               for (i = 0; i < list.GetSize(); i++) {
 #ifdef P_TORNADO
